--------------------------------------------------------------------------------
-- Company      : Fidus Systems
-- Date         : 07/25/2012
-- Designer     : Javier Rodriguez Molina
-- Contact      : j.rodriguez.molina@fidus.com
-- Design Name  : DC1645A 14 bit ADC
-- File Name    : linear_1564a.vhd
-- Hardware     : Stratix IV development board DK-DEV-4SGX230N
-- Description  :
-- This design captures incoming data from the Linear Technology DC1654A-G board.
-- The dual channel 14 bit 310Msps DDR LVDS ADC feeds data to the FPGA. The FPGA
-- captures the 7 bit bus (configurable to 8) and deserializes the stream by a
-- factor of 4 producing 2 samples (even and odd) at half the speed of the
-- incoming data (in this case 155MHz). The captured data from either the odd
-- or even sample is outputed to the HSMC connector on the Stratix development
-- board for testing and it is also captured by Signal Tap. Refer to the
-- DC1564A Design Document for more information
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
library altera;
use altera.altera_syn_attributes.all;
use altera.altera_primitives_components.all;

entity linear_1564a is
  port
  (
    CH_SEL        : in  std_logic                    ;
    CLK_IN        : in  std_logic                    ;
    CLK_OUT       : out std_logic                    ;
    CSn           : out std_logic                    ;
    CSn_in        : in  std_logic                    ;
    SCK           : out std_logic                    ;
    SCK_in        : in  std_logic                    ;
    SDI           : out std_logic                    ;
    SDI_in        : in  std_logic                    ;
    SDO           : in  std_logic                    ;
    SDO_out       : out std_logic                    ;
    DATA_A_IN     : in  std_logic_vector(7 downto 0) ;
    DATA_B_IN     : in  std_logic_vector(7 downto 0) ;
    DATA_OUT      : out std_logic_vector(15 downto 0)
  );

end linear_1564a;

architecture ppl_type of linear_1564a is

--Signal Declaration
signal data_samples_chA : std_logic_vector(31 downto 0);
signal data_samples_chB : std_logic_vector(31 downto 0);
signal data_align_chA   : std_logic_vector(15 downto 0);
signal data_align_chB   : std_logic_vector(15 downto 0);
signal data_out_chA     : std_logic_vector(15 downto 0);
signal data_out_chB     : std_logic_vector(15 downto 0);
signal clk_620mhz       : std_logic;
signal clk_en           : std_logic;
signal pll_locked       : std_logic;
signal clk_155mhz       : std_logic;

--------------------------------------------------------------------------------
--Components
component ext_pll
  PORT
  (
    areset    : IN  STD_LOGIC  := '0';
    inclk0    : IN  STD_LOGIC  := '0';
    c0        : OUT STD_LOGIC        ;
    c1        : OUT STD_LOGIC        ;
    c2        : OUT STD_LOGIC        ;
    locked    : OUT STD_LOGIC
  );
end component;

component lvds_rx_ext_pll
  PORT
  (
   rx_enable      : IN  STD_LOGIC                       ;
   rx_in          : IN  STD_LOGIC_VECTOR (7 DOWNTO 0)   ;
   rx_inclock     : IN  STD_LOGIC                       ;
   rx_out         : OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
  );
end component;

component lvds_tx_pass_thru
  PORT
  (
    tx_in   : IN  STD_LOGIC;
    tx_out  : OUT STD_LOGIC
  );
end component;
--------------------------------------------------------------------------------

begin

--SPI assigment. Modify based on your SPI interface needs
--Refer to the DC1564A Design Document for more information
SCK      <= SCK_in;
SDI      <= SDI_in;
CSn      <= CSn_in;
SDO_out  <= SDO;

--------------------------------------------------------------------------------
--Choose sample to output
--Uncomment EITHER the even or odd sample to output from channel A
--With a deserialization factor of 4 we obtain two 16 bit samples @ 155MHz
--(the 2 LSB are for future used and left are connected to ground)
--------------------------------------------------------------------------------
--Even Sample Channel A
data_align_chA <= ( NOT data_samples_chA(30)&data_samples_chA(31)&
             data_samples_chA(26)&data_samples_chA(27)&
             data_samples_chA(22)&data_samples_chA(23)&
             data_samples_chA(18)&data_samples_chA(19)&
             data_samples_chA(14)&data_samples_chA(15)&
             data_samples_chA(10)&data_samples_chA(11)&
             data_samples_chA(6)&data_samples_chA(7)&
             '0'&'0');                                   --Delete for 16 bit ADC
             --data_samples_chA(2)&data_samples_chA(3)); --Uncomment for 16 bit ADC

--Odd Sample Sample Channel A
--data_align_chA <= (NOT data_samples_chA(28)&data_samples_chA(29)&
--           data_samples_chA(24)&data_samples_chA(27)&
--           data_samples_chA(20)&data_samples_chA(23)&
--           data_samples_chA(16)&data_samples_chA(19)&
--           data_samples_chA(12)&data_samples_chA(15)&
--           data_samples_chA(8)&data_samples_chA(11)&
--           data_samples_chA(4)&data_samples_chA(7)&
--           '0'&'0');                                   --Delete for 16 bit ADC
--           --data_samples_chA(0)&data_samples_chA(1)); --Uncomment for 16 bit ADC

--Even Sample Channel B
data_align_chB <= ( NOT data_samples_chB(30)&data_samples_chB(31)&
             data_samples_chB(26)&data_samples_chB(27)&
             data_samples_chB(22)&data_samples_chB(23)&
             data_samples_chB(18)&data_samples_chB(19)&
             data_samples_chB(14)&data_samples_chB(15)&
             data_samples_chB(10)&data_samples_chB(11)&
             data_samples_chB(6)&data_samples_chB(7)&
             '0'&'0');                                   --Delete for 16 bit ADC
             --data_samples_chB(2)&data_samples_chB(3)); --Uncomment for 16 bit ADC

--Odd Sample Sample Channel B
--data_align_chB <= (NOT data_samples_chB(28)&data_samples_chB(29)&
--           data_samples_chB(24)&data_samples_chB(27)&
--           data_samples_chB(20)&data_samples_chB(23)&
--           data_samples_chB(16)&data_samples_chB(19)&
--           data_samples_chB(12)&data_samples_chB(15)&
--           data_samples_chB(8)&data_samples_chB(11)&
--           data_samples_chB(4)&data_samples_chB(7)&
--           '0'&'0');                                   --Delete for 16 bit ADC
--           --data_samples_chB(0)&data_samples_chB(1)); --Uncomment for 16 bit ADC
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--Instantiate
-- External PLL
ext_pll_inst : ext_pll PORT MAP (
    areset  => open           ,
    inclk0  => CLK_IN         ,
    c0      => clk_620mhz     ,
    c1      => clk_en         ,
    c2      => clk_155mhz     ,
    locked  => pll_locked
  );

--LVDS Receiver Channel A
lvds_receiver_chA : lvds_rx_ext_pll PORT MAP (
  rx_enable   => clk_en           ,
  rx_in       => DATA_A_IN        ,
  rx_inclock  => clk_620mhz       ,
  rx_out      => data_samples_chA
  );
--LVDS Receiver Channel B
lvds_receiver_chB : lvds_rx_ext_pll PORT MAP (
  rx_enable   => clk_en           ,
  rx_in       => DATA_B_IN        ,
  rx_inclock  => clk_620mhz       ,
  rx_out      => data_samples_chB
  );

-- LVDS Transmitter: Output Clock
clk_out_lvds : lvds_tx_pass_thru PORT MAP (
  tx_in       => clk_155mhz      ,
  tx_out      => CLK_OUT
  );
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--Process
--Output Data Assigment -- User Dip Switch SW3.1 -- OFF => chA ON => chB
OUTPUT_DATA: process(clk_155mhz)
  begin
    if (rising_edge(clk_155mhz)) then
		if (CH_SEL = '1') then
			DATA_OUT <= data_out_chA;
		else
			DATA_OUT <= data_out_chB;
		end if;	
    end if;
end process OUTPUT_DATA;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
GEN_LOOP:
  for i in 0 to 15 generate
    data_chA_reg : DFF
      port map (
        d     => data_align_chA(i) ,
        clk   => clk_155mhz        ,
        clrn  => '1'               ,
        prn   => '1'               ,
        q     => data_out_chA(i)
        );

    data_chB_reg : DFF
          port map (
            d     => data_align_chB(i) ,
            clk   => clk_155mhz        ,
            clrn  => '1'               ,
            prn   => '1'               ,
            q     => data_out_chB(i)
            );
  end generate;
--------------------------------------------------------------------------------

end;
